#
# ZPTTidy
#
# Patch to ZopePageTemplates to add html tidy support
#
# Copyright (c) 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#

import re
from HTMLTidy import HTMLTidy
tidy = HTMLTidy()

def strip_ns_nodes(html):
    """
    Remove all nodes (tags and elements) using the metal and tal namespaces.
    Do not remove white-space, esp. line-breaks!
    """
    subs = {
        r'<metal:.+?>'            : '',    # metal opening tag
        r'</metal:.+?>'           : '',    # metal closing tag
        r"(\s+)metal:.+?='.*?'"   : r'\1', # metal attribute with enclosing ''
        r'(\s+)metal:.+?=".*?"'   : r'\1', # metal attribute with enclosing ""
        r'<tal:.+?>'              : '',    # tal opening tag
        r'</tal:.+?>'             : '',    # tal closing tag
        r"(\s+)tal:.+?='.*?'"     : r'\1', # tal attribute with enclosing ''
        r'(\s+)tal:.+?=".*?"'     : r'\1', # tal attribute with enclosing ""
        r'(\s+)xmlns:metal=".*?"' : r'\1', # metal ns attribute with enclosing ''
        r"(\s+)xmlns:metal='.*?'" : r'\1', # metal ns attribute with enclosing ""
        r'(\s+)xmlns:tal=".*?"'   : r'\1', # tal ns attribute with enclosing ''
        r"(\s+)xmlns:tal='.*?'"   : r'\1', # tal ns attribute with enclosing ""
        }

    for key, value in subs.items():
        html = re.sub(key, value, html)

    return html

def tidy_errors(self):
    """
    Analyze template source and return a sequence of dictionaries
    containing the error information
    """
    html = strip_ns_nodes(self.read())
    return tidy.tidyErrors(html)

def pt_tidy(self, REQUEST=None, RESPONSE=None):
    """
    Tidy up
    """
    html = strip_ns_nodes(self.read())          # WARNING: all tal and metal
    html = tidy.tidyUp(html, asxml=1, indent=1) # information will be lost!
    if html is None:
        from Globals import MessageDialog
        message = "HTML source contains errors. Tidying is therefore not possible. Please fix these errors first."
        return MessageDialog(title='Tidying not possible', message=message, action='pt_tidyForm')

    self.write(html)
    if REQUEST is not None:
        message = "Tidied HTML source."
        return self.pt_editForm(REQUEST, manage_tabs_message=message)

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
pt_tidyForm = PageTemplateFile("ptTidy.zpt", globals())

__ac_permissions__ = (
    ("View management screens", ("tidy_errors", "pt_tidy", "pt_tidyForm") ),
    )

# the patch will only be applied if the tidy program is available
if tidy.checkTidy():
    from Globals import InitializeClass
    from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

    # Monkey-patch the original class
    ZopePageTemplate.tidy_errors        = tidy_errors
    ZopePageTemplate.pt_tidy            = pt_tidy
    ZopePageTemplate.pt_tidyForm        = pt_tidyForm
    ZopePageTemplate.__ac_permissions__ = __ac_permissions__
    ZopePageTemplate.manage_options     = (
        ZopePageTemplate.manage_options[:2] + \
        ({'label': 'Tidy', 'action': 'pt_tidyForm' },) + \
        ZopePageTemplate.manage_options[2:]
        )

    # re-initalize the class so the security declarations will be updated
    InitializeClass(ZopePageTemplate)
